import { Client, Events, GatewayIntentBits, Guild, GuildBasedChannel, Interaction } from "discord.js"
import { utilities } from "./utilties"
import { commands } from "./commands"
import { config } from "./config"
import { SQLCommon } from "./utilties/storage/interfaces"

export const client : Client = new Client({
    intents: [
        GatewayIntentBits.Guilds,
        GatewayIntentBits.GuildMessages,
        GatewayIntentBits.DirectMessages
    ]
})

let db: SQLCommon

if (config.DB_MODE == "sqlite") {
    db = new utilities.sqlite.SqliteDB("breadbot_test.db")

    db.run("PRAGMA foreign_keys = ON")

    utilities.tables.makeTables(db)
    utilities.tables.makeConstraints(db)
}

client.once(Events.ClientReady, () => {
    // TODO Winston should handle this
    console.log("Breadbot is ready")

    client.guilds.cache.forEach(async (guild: Guild) => {
        await utilities.commands.deployCommands(guild.id)

        // TODO handle failures?
        await utilities.guilds.insertGuild(db, guild)

        guild.channels.cache.forEach(async (channel: GuildBasedChannel) => {
            await utilities.channels.insertChannel(db, channel)
        })
    })
})

client.on(Events.GuildCreate, async (guild : Guild) => {
    await utilities.commands.deployCommands(guild.id)
    await utilities.guilds.insertGuild(db, guild)

    guild.channels.cache.forEach(async (channel: GuildBasedChannel) => {
        await utilities.channels.insertChannel(db, channel)
    })
})

client.on(Events.InteractionCreate, async (interaction: Interaction) => {
    if (!interaction.isCommand()) {
        return
    }

    if (commands[interaction.commandName as keyof typeof commands]) {
        commands[interaction.commandName as keyof typeof commands].execute(interaction)
    }
})

client.login(config.DISCORD_TOKEN)