import { SQLCommon } from "../storage/interfaces";
import { GuildBasedChannel } from "discord.js";
import { SQLResult } from "../storage/enumerations";

export async function doesChannelExist(db: SQLCommon, channel : GuildBasedChannel) : Promise<boolean> {
    const queryResult : Object[] = await db.getAllParameterized(
        "SELECT * FROM channels WHERE server_snowflake = ? AND channel_snowflake = ?",
        [channel.guild.id, channel.id]
    )

    return queryResult.length != 0
}

export async function insertChannel(db: SQLCommon, channel: GuildBasedChannel) : Promise<SQLResult> {
    const alreadyExists: boolean = await doesChannelExist(db, channel)

    if(alreadyExists) {
        return SQLResult.ALREADYEXISTS
    }

    try {
        await db.runParameterized(
            "INSERT INTO channels VALUES (?, ?, ?, ?)",
            [channel.id, channel.guild.id, channel.name, channel.isThread()]
        )

        return SQLResult.CREATED
    } catch (err) {
        //TODO Winston should handle this
        console.log(err)
        return SQLResult.FAILED
    }
}